{ This unit defines the structure of the model. There are four functions. The
  first function, called counts, defines the number, names, and units of the
  model, the state variables, the process variables, the driver variables and
  the parameters. The second function, called processes, is the actual equations
  which make up the model. The third function, derivs, calculates the
  derivatives of state variables. And the fourth function, parcount, is used to
  automatically number the parameters consecutively. 
    The state variables, driver variables, process variables and parameters are
  all stored in global arrays, called stat, drive, proc, and par, respectively.
  The function counts accesses the global arrays directly but the other functions
  operate on copies of the global arrays. }
unit equations;

interface

uses  stypes, math;

PROCEDURE counts;
PROCEDURE processes(time:double; dtime:double; var tdrive:drivearray;
                       var tpar:paramarray; var tstat:statearray;
                       var tproc:processarray; CalculateDiscrete:Boolean);
PROCEDURE derivs(t, drt:double; var tdrive:drivearray; var tpar:paramarray;
             var statevalue:yValueArray; VAR dydt:yValueArray);
function ParCount(processnum:integer) : integer;

var
  tproc:processarray;
  tstat:statearray;
  sensflag:boolean;
  newyear:Boolean = false;
  DayofYear: double = 0;

implementation

uses frontend, calculate, options;

           { Do not make modifcations above this line. }
{*****************************************************************************}

{ This procedure defines the model. The number of parameters, state, driver and
  process variables are all set in this procedure. The model name, version
  number and time unit are also set here. This procedure accesses the global
  arrays containing the the parameters, state, driver and process variables and
  the global structure ModelDef directly, to save memory space. }
PROCEDURE counts;
var
 i,npar,CurrentProc:integer;
begin
{ Set the modelname, version and time unit. }
ModelDef.modelname := 'Frontiers Models';
ModelDef.versionnumber := '1.0.0';
ModelDef.timeunit := 'years';
ModelDef.contactperson := 'Edward B. Rastetter';
ModelDef.contactaddress1 := 'The Ecosystems Center';
ModelDef.contactaddress2 := 'Marine Biological Laboratory';
ModelDef.contactaddress3 := 'Woods Hole, MA 02543';

{ Set the number of state variables in the model. The maximum number of state
  variables is maxstate, in unit stypes. }
ModelDef.numstate := 11;

{ Enter the name, units and symbol for each state variable. The maximum length
  of the state variable name is 17 characters and the maximum length for units
  and symbol is stringlength (specified in unit stypes) characters. }
 
 
with stat[1] do
 begin
    name:='plant C';  units:='g C m-2';  symbol:='BC';
 end;
 
with stat[2] do
 begin
    name:='plant N';  units:='g N m-2';  symbol:='BN';
 end;
 
with stat[3] do
 begin
    name:='plant P';  units:='g P m-2';  symbol:='BP';
 end;
 
with stat[4] do
 begin
    name:='Detrital C';  units:='g C m-2';  symbol:='DC';
 end;
 
with stat[5] do
 begin
    name:='Detrital N';  units:='g N m-2';  symbol:='DN';
 end;
 
with stat[6] do
 begin
    name:='Detrital P';  units:='g P m-2';  symbol:='DP';
 end;
 
with stat[7] do
 begin
    name:='Inorganic N';  units:='g N m-2';  symbol:='EN';
 end;
 
with stat[8] do
 begin
    name:='Inorganic P';  units:='g P m-2';  symbol:='EP';
 end;
 
with stat[9] do
 begin
    name:='effort C';  units:='none';  symbol:='VC';
 end;
 
with stat[10] do
 begin
    name:='effort N';  units:='none';  symbol:='VN';
 end;
 
with stat[11] do
 begin
    name:='effort P';  units:='none';  symbol:='VP';
 end;

{ Set the total number of processes in the model. The first numstate processes
  are the derivatives of the state variables. The maximum number of processes is
  maxparam, in unit stypes. }
ModelDef.numprocess := ModelDef.numstate + 49;

{ For each process, set proc[i].parameters equal to the number of parameters
  associated with that process, and set IsDiscrete to true or false. After each
  process, set the name, units, and symbol for all parameters associated with
  that process. Note that Parcount returns the total number of parameters in
  all previous processes. }
 
CurrentProc := ModelDef.numstate + 1;
With proc[CurrentProc] do
   begin
      name       := 'NPP';
      units       := 'g C m-2 year-1';
      symbol       := 'PN';
      parameters       := 2;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Max NPP rate';  units:='g C m-2 year-1';  symbol:='Pmax';
 end;
with par[npar + 2] do
 begin
    name:='CO2 half sat';  units:='ppm';  symbol:='KC';
 end;
 
CurrentProc := ModelDef.numstate + 2;
With proc[CurrentProc] do
   begin
      name       := 'Plant N uptake';
      units       := 'g N m-2 year-1';
      symbol       := 'UN';
      parameters       := 2;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Max N up rate';  units:='g C m-2 year-1';  symbol:='UNmax';
 end;
with par[npar + 2] do
 begin
    name:='Plant N half sat';  units:='g N m-2';  symbol:='KN';
 end;
 
CurrentProc := ModelDef.numstate + 3;
With proc[CurrentProc] do
   begin
      name       := 'Plant P uptake';
      units       := 'g P m-2 year-1';
      symbol       := 'UP';
      parameters       := 2;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Max P up rate';  units:='g C m-2 year-1';  symbol:='UPmax';
 end;
with par[npar + 2] do
 begin
    name:='Plant P half sat';  units:='g P m-2';  symbol:='KP';
 end;
 
CurrentProc := ModelDef.numstate + 4;
With proc[CurrentProc] do
   begin
      name       := 'Net C uptake';
      units       := 'g C m-2 year-1';
      symbol       := 'UnC';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 5;
With proc[CurrentProc] do
   begin
      name       := 'Net N Uptake';
      units       := 'g N m-2 year-1';
      symbol       := 'UnN';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 6;
With proc[CurrentProc] do
   begin
      name       := 'Net P Uptake';
      units       := 'g P m-2 year-1';
      symbol       := 'UnP';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 7;
With proc[CurrentProc] do
   begin
      name       := 'Plant litter C';
      units       := 'g C m-2 year-1';
      symbol       := 'L_C';
      parameters       := 2;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Active turnover const';  units:='year-1';  symbol:='aA';
 end;
with par[npar + 2] do
 begin
    name:='Wood turnover const';  units:='year-1';  symbol:='aW';
 end;
 
CurrentProc := ModelDef.numstate + 8;
With proc[CurrentProc] do
   begin
      name       := 'Plant litter N';
      units       := 'g N m-2 year-1';
      symbol       := 'L_N';
      parameters       := 2;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Actve litter C:N';  units:='g C g-1 N';  symbol:='qLNA';
 end;
with par[npar + 2] do
 begin
    name:='Wood litter C:N';  units:='g C g-1 N';  symbol:='qLNW';
 end;
 
CurrentProc := ModelDef.numstate + 9;
With proc[CurrentProc] do
   begin
      name       := 'Plant litter P';
      units       := 'g P m-2 year-1';
      symbol       := 'L_P';
      parameters       := 2;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Actve litter C:P';  units:='g C g-1 P';  symbol:='qLPA';
 end;
with par[npar + 2] do
 begin
    name:='Wood litter C:P';  units:='g C g-1 P';  symbol:='qLPW';
 end;
 
CurrentProc := ModelDef.numstate + 10;
With proc[CurrentProc] do
   begin
      name       := 'microbial respiration';
      units       := 'g C m-2 year-1';
      symbol       := 'Rm';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 11;
With proc[CurrentProc] do
   begin
      name       := 'Gross N mineralization';
      units       := 'g N m-2 year-1';
      symbol       := 'MN';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 12;
With proc[CurrentProc] do
   begin
      name       := 'Gross P mineralization';
      units       := 'g P m-2 year-1';
      symbol       := 'MP';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 13;
With proc[CurrentProc] do
   begin
      name       := 'microbial DC use';
      units       := 'g C m-2 yr-1';
      symbol       := 'GmC';
      parameters       := 4;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Soil C trunover const';  units:='year -1';  symbol:='rmC';
 end;
with par[npar + 2] do
 begin
    name:='Soil N trunover const';  units:='year -1';  symbol:='rmN';
 end;
with par[npar + 3] do
 begin
    name:='Soil P trunover const';  units:='year -1';  symbol:='rmP';
 end;
with par[npar + 4] do
 begin
    name:='max microb C efficiency';  units:='fraction';  symbol:='epsilonCx';
 end;
 
CurrentProc := ModelDef.numstate + 14;
With proc[CurrentProc] do
   begin
      name       := 'microbial C efficiency';
      units       := 'fraction';
      symbol       := 'epsilonmC';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 15;
With proc[CurrentProc] do
   begin
      name       := 'microbial N efficiency';
      units       := 'fraction';
      symbol       := 'epsilonmN';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 16;
With proc[CurrentProc] do
   begin
      name       := 'microbial P efficiency';
      units       := 'fraction';
      symbol       := 'epsilonmP';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 17;
With proc[CurrentProc] do
   begin
      name       := 'microbial DN use';
      units       := 'g C m-2 yr-1';
      symbol       := 'GmN';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 18;
With proc[CurrentProc] do
   begin
      name       := 'microbial DP use';
      units       := 'g C m-2 yr-1';
      symbol       := 'GmP';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 19;
With proc[CurrentProc] do
   begin
      name       := 'N immobilization';
      units       := 'g N m-2 year-1';
      symbol       := 'UmN';
      parameters       := 3;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='N immob rate const';  units:='year -1';  symbol:='betaN';
 end;
with par[npar + 2] do
 begin
    name:='microb N half sat';  units:='g N m-2';  symbol:='KmN';
 end;
with par[npar + 3] do
 begin
    name:='microb C:N';  units:='g C g-1 N';  symbol:='qmN';
 end;
 
CurrentProc := ModelDef.numstate + 20;
With proc[CurrentProc] do
   begin
      name       := 'P immobilization';
      units       := 'g P m-2 year-1';
      symbol       := 'UmP';
      parameters       := 3;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='P immob rate const';  units:='year -1';  symbol:='betaP';
 end;
with par[npar + 2] do
 begin
    name:='microb P half sat';  units:='g P m-2';  symbol:='KmP';
 end;
with par[npar + 3] do
 begin
    name:='microb C:P';  units:='g C g-1 P';  symbol:='qmP';
 end;
 
CurrentProc := ModelDef.numstate + 21;
With proc[CurrentProc] do
   begin
      name       := 'Net N min';
      units       := 'g N m-2 year-1';
      symbol       := 'Nmin';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 22;
With proc[CurrentProc] do
   begin
      name       := 'Net P min';
      units       := 'g P m-2 year-1';
      symbol       := 'Pmin';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 23;
With proc[CurrentProc] do
   begin
      name       := 'Organic C loss';
      units       := 'g C m-2 year-1';
      symbol       := 'QOC';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='DOC loss const';  units:='year-1';  symbol:='alphaOC';
 end;
 
CurrentProc := ModelDef.numstate + 24;
With proc[CurrentProc] do
   begin
      name       := 'Organic N loss';
      units       := 'g N m-2 year-1';
      symbol       := 'QON';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='DON loss const';  units:='year-1';  symbol:='alphaON';
 end;
 
CurrentProc := ModelDef.numstate + 25;
With proc[CurrentProc] do
   begin
      name       := 'Organic P loss';
      units       := 'g P m-2 year-1';
      symbol       := 'QOP';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='DOP loss const';  units:='year-1';  symbol:='alphaOP';
 end;
 
CurrentProc := ModelDef.numstate + 26;
With proc[CurrentProc] do
   begin
      name       := 'Inorganic N loss';
      units       := 'g N m-2 year-1';
      symbol       := 'QN';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='DIN loss const';  units:='year-1';  symbol:='alphaN';
 end;
 
CurrentProc := ModelDef.numstate + 27;
With proc[CurrentProc] do
   begin
      name       := 'Inorganic P loss';
      units       := 'g P m-2 year-1';
      symbol       := 'QP';
      parameters       := 1;
      ptype       := ptGroup1;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='DIP loss const';  units:='year-1';  symbol:='alphaP';
 end;
 
CurrentProc := ModelDef.numstate + 28;
With proc[CurrentProc] do
   begin
      name       := 'Active biomass';
      units       := 'g C m-2';
      symbol       := 'BA';
      parameters       := 2;
      ptype       := ptGroup2;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='maximum active biom';  units:='g c m-2';  symbol:='BAmax';
 end;
with par[npar + 2] do
 begin
    name:='allometric const';  units:='none';  symbol:='gammaB';
 end;
 
CurrentProc := ModelDef.numstate + 29;
With proc[CurrentProc] do
   begin
      name       := 'woody biomass';
      units       := 'g C m-2';
      symbol       := 'BW';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 30;
With proc[CurrentProc] do
   begin
      name       := 'Opt Plant C:N';
      units       := 'g C g-1N';
      symbol       := 'q_N';
      parameters       := 2;
      ptype       := ptGroup2;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='C:N active biom';  units:='g C g-1 N';  symbol:='qNA';
 end;
with par[npar + 2] do
 begin
    name:='C:N wood';  units:='g C g-1 N';  symbol:='qNW';
 end;
 
CurrentProc := ModelDef.numstate + 31;
With proc[CurrentProc] do
   begin
      name       := 'Opt Plant C:P';
      units       := 'g C g-1P';
      symbol       := 'q_P';
      parameters       := 2;
      ptype       := ptGroup2;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='C:P active biom';  units:='g C g-1 P';  symbol:='qPA';
 end;
with par[npar + 2] do
 begin
    name:='C:P wood';  units:='g C g-1 P';  symbol:='qPW';
 end;
 
CurrentProc := ModelDef.numstate + 32;
With proc[CurrentProc] do
   begin
      name       := 'Plant C:N';
      units       := 'g C g-1N';
      symbol       := 'AqN';
      parameters       := 0;
      ptype       := ptGroup2;
   end;
 
CurrentProc := ModelDef.numstate + 33;
With proc[CurrentProc] do
   begin
      name       := 'Plant C:P';
      units       := 'g C g-1N';
      symbol       := 'AqP';
      parameters       := 0;
      ptype       := ptGroup2;
   end;
 
CurrentProc := ModelDef.numstate + 34;
With proc[CurrentProc] do
   begin
      name       := 'Litter C:N';
      units       := 'g C g-1N';
      symbol       := 'LqN';
      parameters       := 0;
      ptype       := ptGroup2;
   end;
 
CurrentProc := ModelDef.numstate + 35;
With proc[CurrentProc] do
   begin
      name       := 'Litter C:P';
      units       := 'g C g-1N';
      symbol       := 'LqP';
      parameters       := 0;
      ptype       := ptGroup2;
   end;
 
CurrentProc := ModelDef.numstate + 36;
With proc[CurrentProc] do
   begin
      name       := 'Requirement C';
      units       := 'g C m-2 year-1';
      symbol       := 'RC';
      parameters       := 0;
      ptype       := ptGroup5;
   end;
 
CurrentProc := ModelDef.numstate + 37;
With proc[CurrentProc] do
   begin
      name       := 'Requirement N';
      units       := 'g N m-2 year-1';
      symbol       := 'RN';
      parameters       := 0;
      ptype       := ptGroup5;
   end;
 
CurrentProc := ModelDef.numstate + 38;
With proc[CurrentProc] do
   begin
      name       := 'Requirement P';
      units       := 'g P m-2 year-1';
      symbol       := 'RP';
      parameters       := 0;
      ptype       := ptGroup5;
   end;
 
CurrentProc := ModelDef.numstate + 39;
With proc[CurrentProc] do
   begin
      name       := 'Mean Up:Req';
      units       := 'none';
      symbol       := 'PSI';
      parameters       := 1;
      ptype       := ptGroup5;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='acclim rate';  units:='year-1';  symbol:='a';
 end;
 
CurrentProc := ModelDef.numstate + 40;
With proc[CurrentProc] do
   begin
      name       := 'symbiotic N fixation';
      units       := 'g N m-2 year-1';
      symbol       := 'UNfix';
      parameters       := 3;
      ptype       := ptGroup2;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='max symb N fix';  units:='g N g-1 C yr-1';  symbol:='FNF';
 end;
with par[npar + 2] do
 begin
    name:='BA at canopy closure';  units:='fraction';  symbol:='BNF';
 end;
with par[npar + 3] do
 begin
    name:='canopy inhibition';  units:='m2 g-1 C';  symbol:='kNF';
 end;
 
CurrentProc := ModelDef.numstate + 41;
With proc[CurrentProc] do
   begin
      name       := 'stoichio net N growth';
      units       := 'g N m-2 yr-1';
      symbol       := 'PsiN';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 42;
With proc[CurrentProc] do
   begin
      name       := 'fixation potential';
      units       := 'g N m-2 yr-1';
      symbol       := 'FNP';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 43;
With proc[CurrentProc] do
   begin
      name       := 'non-symbiotic N fixatio';
      units       := 'g N m-2 year-1';
      symbol       := 'INF';
      parameters       := 1;
      ptype       := ptGroup2;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='N-S fixation constant';  units:='g N g-2/3 C m-2/3 yr-1';  symbol:='gammaNF';
 end;
 
CurrentProc := ModelDef.numstate + 44;
With proc[CurrentProc] do
   begin
      name       := 'Growth N:C';
      units       := 'gN g-1 C';
      symbol       := 'phiN';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 45;
With proc[CurrentProc] do
   begin
      name       := 'Growth P:C';
      units       := 'gP g-1 C';
      symbol       := 'phiP';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 46;
With proc[CurrentProc] do
   begin
      name       := 'Model options';
      units       := 'none';
      symbol       := 'MODEL';
      parameters       := 13;
      ptype       := ptGroup5;
   end;
npar:=ParCount(CurrentProc);
with par[npar + 1] do
 begin
    name:='Plant flag';  units:='0=UnC 1=Lib 2=Conc 3=Accl';  symbol:='flag1';
 end;
with par[npar + 2] do
 begin
    name:='Microb flag';  units:='0=UnC 1=Lib 2=Conc 3=Accl';  symbol:='flag2';
 end;
with par[npar + 3] do
 begin
    name:='Plant mult weighting';  units:='g C m2 yr2 g-1 Ng-1P';  symbol:='rhoM';
 end;
with par[npar + 4] do
 begin
    name:='Microb mult weighting';  units:='m4 yr2 g-1N g-1 P';  symbol:='rhomM';
 end;
with par[npar + 5] do
 begin
    name:='Q10 NPP';  units:='none';  symbol:='Q10Pn';
 end;
with par[npar + 6] do
 begin
    name:='Q10 plant N up';  units:='none';  symbol:='Q10UN';
 end;
with par[npar + 7] do
 begin
    name:='Q10 plant P up';  units:='none';  symbol:='Q10UP';
 end;
with par[npar + 8] do
 begin
    name:='Q10 Nfix';  units:='none';  symbol:='Q10Nfix';
 end;
with par[npar + 9] do
 begin
    name:='Q10 Soil C turnover';  units:='none';  symbol:='Q10DC';
 end;
with par[npar + 10] do
 begin
    name:='Q10 Soil N turnover';  units:='none';  symbol:='Q10DN';
 end;
with par[npar + 11] do
 begin
    name:='Q10 Soil P turnover';  units:='none';  symbol:='Q10DP';
 end;
with par[npar + 12] do
 begin
    name:='Q10 microbe N up';  units:='none';  symbol:='Q10mUN';
 end;
with par[npar + 13] do
 begin
    name:='Q10 microbe P up';  units:='none';  symbol:='Q10mUP';
 end;
 
CurrentProc := ModelDef.numstate + 47;
With proc[CurrentProc] do
   begin
      name       := 'microb effort C';
      units       := 'none';
      symbol       := 'etaC';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 48;
With proc[CurrentProc] do
   begin
      name       := 'microb effort N';
      units       := 'none';
      symbol       := 'etaN';
      parameters       := 0;
      ptype       := ptGroup1;
   end;
 
CurrentProc := ModelDef.numstate + 49;
With proc[CurrentProc] do
   begin
      name       := 'microb effort P';
      units       := 'none';
      symbol       := 'etaP';
      parameters       := 0;
      ptype       := ptGroup1;
   end;

{ Set the total number of drivers in the model. The maximum number of drivers is
  maxdrive, in unit stypes. }
ModelDef.numdrive := 4;

{ Set the names, units, and symbols of the drivers. The maximum length for the
  name, units and symbol is 20 characters. }
 
with drive[1] do
 begin
    name:='CO2';  units:='MJ m-2 year-1';  symbol:='CO2';
 end;
 
with drive[2] do
 begin
    name:='N deposition';  units:='g N m-2 year-1';  symbol:='I_N';
 end;
 
with drive[3] do
 begin
    name:='P inputs';  units:='g P m-2 year-1';  symbol:='I_P';
 end;
 
with drive[4] do
 begin
    name:='temperature';  units:='oC';  symbol:='Temp';
 end;

{ The first numstate processes are the derivatives of the state variables. The
  code sets the names, units and symbols accordingly.}
for i:= 1 to ModelDef.numstate do proc[i].name:='d'+stat[i].name+'dt';
for i:= 1 to ModelDef.numstate do proc[i].units := stat[i].units + 't-1';
for i:= 1 to ModelDef.numstate do proc[i].symbol := 'd' + stat[i].symbol + 'dt';

{ Code to sum up the total number of parameters in the model. Do not change the
  next few lines. }
ModelDef.numparam := 0;
for i := 1 to ModelDef.NumProcess do
  ModelDef.numparam := ModelDef.numparam + proc[i].parameters;

end; // counts procedure


{ A procedure to calculate the value of all states and processes at the current
  time. This function accesses time, state variables and process variables by
  reference, ie it uses the same array as the calling routine. It does not use
  the global variables time, stat and proc because values calculated during
  integration might later be discarded. It does access the global variables par,
  drive and ModelDef directly because those values are not modified.

  The model equations are written using variable names which correspond to the
  actual name instead of using the global arrays (i.e. SoilWater instead of
  stat[7].value). This makes it necessary to switch all values into local
  variables, do all the calculations and then put everything back into the
  global variables. Lengthy but worth it in terms of readability of the code. }

// Choose either GlobalPs, ArcticPs, or none here so the appropriate Ps model is compiled below.
{$DEFINE none}

PROCEDURE processes(time:double; dtime:double; var tdrive:drivearray;
                       var tpar:paramarray; var tstat:statearray;
                       var tproc:processarray; CalculateDiscrete:Boolean);
{$IFDEF GlobalPs}
const
// Global Ps parameters
 x1 = 11.04;             x2 = 0.03;
 x5 = 0.216;             x6 = 0.6;
 x7 = 3.332;             x8 = 0.004;
 x9 = 1.549;             x10 = 1.156;
 gammastar = 0;          kCO2 = 995.4;  }
{$ENDIF}

// Modify constant above (line above "procedure processes..." line )to specify
// which Ps model and it's constants should be compiled. Choosing a Ps model
// automatically includes the Et and Misc constants (i.e. Gem is assumed).

{$IFDEF ArcticPs}
const
// Arctic Ps parameters
x1 = 0.192;	x2 = 0.125;
x5 = 2.196;	x6 = 50.41;
x7 = 0.161;	x8 = 14.78;
x9 = 1.146;
gammastar = 0.468;	kCO2 = 500.3;
{$ENDIF}

{$IFDEF ArcticPs OR GlobalPs}
//const
// General Et parameters
aE1 = 0.0004;    aE2 = 150;  aE3 = 1.21;   aE4 = 6.11262E5;

// Other constants
cp = 1.012E-9; //specific heat air MJ kg-1 oC-1
sigmaSB = 4.9e-9; //stefan-Boltzmann MJ m-2 day-1 K-4
S0 = 117.5; //solar constant MJ m-2 day-1
bHI1 =0.23;
bHI2 =0.48;
mw = 2.99; //kg h2o MJ-1
alphaMS = 2; //mm oC-1 day-1                                 }
{$ENDIF}

var
{ List the variable names you are going to use here. Generally, this list
  includes all the symbols you defined in the procedure counts above. The order
  in which you list them does not matter. }
{States}
BC, dBCdt, 
BN, dBNdt, 
BP, dBPdt, 
DC, dDCdt, 
DN, dDNdt, 
DP, dDPdt, 
EN, dENdt, 
EP, dEPdt, 
VC, dVCdt, 
VN, dVNdt, 
VP, dVPdt, 

{processes and associated parameters}
PN, Pmax, KC, 
UN, UNmax, KN, 
UP, UPmax, KP, 
UnC, 
UnN, 
UnP, 
L_C, aA, aW, 
L_N, qLNA, qLNW, 
L_P, qLPA, qLPW, 
Rm, 
MN, 
MP, 
GmC, rmC, rmN, rmP, epsilonCx, 
epsilonmC, 
epsilonmN, 
epsilonmP, 
GmN, 
GmP, 
UmN, betaN, KmN, qmN, 
UmP, betaP, KmP, qmP, 
Nmin, 
Pmin, 
QOC, alphaOC, 
QON, alphaON, 
QOP, alphaOP, 
QN, alphaN, 
QP, alphaP, 
BA, BAmax, gammaB, 
BW, 
q_N, qNA, qNW, 
q_P, qPA, qPW, 
AqN, 
AqP, 
LqN, 
LqP, 
RC, 
RN, 
RP, 
PSI, a, 
UNfix, FNF, BNF, kNF, 
PsiN, 
FNP, 
INF, gammaNF, 
phiN, 
phiP, 
MODEL, flag1, flag2, rhoM, rhomM, Q10Pn, Q10UN, Q10UP, Q10Nfix, Q10DC, Q10DN, Q10DP, Q10mUN, Q10mUP, 
etaC, 
etaN, 
etaP, 

{drivers}
CO2, 
I_N, 
I_P, 
Temp, 

{Other double}
dum, 
UmC

:double; {Final double}
{Other integers}
npar, j, jj, kk, ll, tnum:integer;


{ Functions or procedures }

begin
{ Copy the drivers from the global array, drive, into the local variables. }
CO2 := tdrive[1].value;
I_N := tdrive[2].value;
I_P := tdrive[3].value;
Temp := tdrive[4].value;

{ Copy the state variables from the global array into the local variables. }
BC := tstat[1].value;
BN := tstat[2].value;
BP := tstat[3].value;
DC := tstat[4].value;
DN := tstat[5].value;
DP := tstat[6].value;
EN := tstat[7].value;
EP := tstat[8].value;
VC := tstat[9].value;
VN := tstat[10].value;
VP := tstat[11].value;

{ And now copy the parameters into the local variables. No need to copy the
  processes from the global array into local variables. Process values will be
  calculated by this procedure.

  Copy the parameters for each process separately using the function ParCount
  to keep track of the number of parameters in the preceeding processes.
  npar now contains the number of parameters in the preceding processes.
  copy the value of the first parameter of this process into it's local
  variable }
npar:=ParCount(ModelDef.numstate + 1);
Pmax := par[npar + 1].value;
KC := par[npar + 2].value;

npar:=ParCount(ModelDef.numstate + 2);
UNmax := par[npar + 1].value;
KN := par[npar + 2].value;
 
npar:=ParCount(ModelDef.numstate + 3);
UPmax := par[npar + 1].value;
KP := par[npar + 2].value;
 
npar:=ParCount(ModelDef.numstate + 7);
aA := par[npar + 1].value;
aW := par[npar + 2].value;
 
npar:=ParCount(ModelDef.numstate + 8);
qLNA := par[npar + 1].value;
qLNW := par[npar + 2].value;
 
npar:=ParCount(ModelDef.numstate + 9);
qLPA := par[npar + 1].value;
qLPW := par[npar + 2].value;
 
npar:=ParCount(ModelDef.numstate + 13);
rmC := par[npar + 1].value;
rmN := par[npar + 2].value;
rmP := par[npar + 3].value;
epsilonCx := par[npar + 4].value;
 
npar:=ParCount(ModelDef.numstate + 19);
betaN := par[npar + 1].value;
KmN := par[npar + 2].value;
qmN := par[npar + 3].value;
 
npar:=ParCount(ModelDef.numstate + 20);
betaP := par[npar + 1].value;
KmP := par[npar + 2].value;
qmP := par[npar + 3].value;
 
npar:=ParCount(ModelDef.numstate + 23);
alphaOC := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 24);
alphaON := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 25);
alphaOP := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 26);
alphaN := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 27);
alphaP := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 28);
BAmax := par[npar + 1].value;
gammaB := par[npar + 2].value;
 
npar:=ParCount(ModelDef.numstate + 30);
qNA := par[npar + 1].value;
qNW := par[npar + 2].value;
 
npar:=ParCount(ModelDef.numstate + 31);
qPA := par[npar + 1].value;
qPW := par[npar + 2].value;
 
npar:=ParCount(ModelDef.numstate + 39);
a := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 40);
FNF := par[npar + 1].value;
BNF := par[npar + 2].value;
kNF := par[npar + 3].value;
 
npar:=ParCount(ModelDef.numstate + 43);
gammaNF := par[npar + 1].value;
 
npar:=ParCount(ModelDef.numstate + 46);
flag1 := par[npar + 1].value;
flag2 := par[npar + 2].value;
rhoM := par[npar + 3].value;
rhomM := par[npar + 4].value;
Q10Pn := par[npar + 5].value;
Q10UN := par[npar + 6].value;
Q10UP := par[npar + 7].value;
Q10Nfix := par[npar + 8].value;
Q10DC := par[npar + 9].value;
Q10DN := par[npar + 10].value;
Q10DP := par[npar + 11].value;
Q10mUN := par[npar + 12].value;
Q10mUP := par[npar + 13].value;
 
 
{ Enter the equations to calculate the processes here, using the local variable
  names defined above. }


MODEL:=flag1;
if EN<0  then    EN:=0;
if EP<0  then    EP:=0;

dum:=VC+VN+VP;
VC:=VC/dum;
VN:=VN/dum;
VP:=VP/dum;

AqN:=BC/BN;
AqP:=BC/BP;

BA:=BAmax*gammaB*BC/(BAmax+gammaB*BC);
BW:=BC-BA;

q_N:=BC/(BA/qNA + BW/qNW);
q_P:=BC/(BA/qPA + BW/qPW);

PN:=power(Q10Pn,(Temp-15)/10)*BA*Pmax*CO2/(kC+CO2);
UN:=power(Q10UN,(Temp-15)/10)*BA*UNmax*EN/(kN+EN);
UP:=power(Q10UP,(Temp-15)/10)*BA*UPmax*EP/(kP+EP);

L_C:=aA*BA + aW*BW;
L_N:=(aA*BA/qLNA + aW*BW/qLNW)*BN*q_N/BC;
L_P:=(aA*BA/qLPA + aW*BW/qLPW)*BP*q_P/BC;

LqN:=L_C/L_N;
LqP:=L_C/L_P;


phiN:=sqr(BA)*(1/qNA-1/qNW)/(gammaB*sqr(BC))+1/qNW;
phiP:=sqr(BA)*(1/qPA-1/qPW)/(gammaB*sqr(BC))+1/qPW;

UnC:=PN-L_C;
UnP:=UP-L_P;

PsiN:=min(phiN*UnC, phiN*UnP/phiP);
FNP:=max(0, PsiN-Un+L_N);
UNfix:=power(Q10Nfix,(Temp-15)/10)*FNF*BA*FNP/(1+exp(kNF*(BA-BNF)));
UnN:=UN+UNfix-L_N;


if BW>0 then INF:=gammaNF*power(BW,2/3) else INF:=0;

RC:=L_C*sqrt(sqrt(q_N*BN*q_P*BP/sqr(BC)));
RN:=L_N*sqrt(BC/(BN*q_N));
RP:=L_P*sqrt(BC/(BP*q_P));
PSI:=Power(PN/RC,VC)*Power((UN+UNfix)/RN,VN)*Power(UP/RP,VP);

dVCdt:=0;
dVNdt:=0;
dVPdt:=0;


if abs(flag1-1)<0.1 then {Liebig}
   begin
      UnC:=min(UnC,min(UnN/phiN,UnP/phiP));
      UnN:=UnC *phiN;
      UnP:=UnC *phiP;
      PN:=UnC+L_C;
      UNfix:=max(0,UnN-UN+L_N);
      UN:=UnN+L_N-UNfix;
      UP:=UnP+L_P;
   end;
if abs(flag1-2)<0.1 then  {concurrent}
   begin
      Pn:=rhoM*Pn*(UN+UNfix)*UP/sqr(BA);
      UnC:=Pn-L_C;
      UnN:=UnC *phiN;
      UnP:=UnC *phiP;      
      UN:=max(0,UnN-UNfix+L_N);
      UNfix:=UnN+L_N-UN;
      UP:=UnP+L_P;
   end;
if abs(flag1-3)<0.1 then  {adaptive}
   begin
      PN:=3*VC*PN;
      UN:=3*VN*UN;
      UNfix:=3*VN*UNfix;
      UP:=3*VP*UP;

      UnC:=PN-L_C;
      UnN:=UN+UNfix-L_N;
      UnP:=UP-L_P;

      if PSI>0 then
         begin
            dVCdt:=a*ln(PSI*RC/PN)*VC;
            dVNdt:=a*ln(PSI*RN/(UN+UNfix))*VN;
            dVPdt:=a*ln(PSI*RP/UP)*VP;
         end;
   end;

GmC:=power(Q10DC,(Temp-15)/10)*rmC*DC;
GmN:=power(Q10DN,(Temp-15)/10)*rmN*DN;
GmP:=power(Q10DP,(Temp-15)/10)*rmP*DP;

UmN:=power(Q10mUN,(Temp-15)/10)*betaN*DC*(DC/(qmN*DN))*EN/(kmN+EN);
UmP:=power(Q10mUP,(Temp-15)/10)*betaP*DC*(DC/(qmP*DP))*EP/(kmP+EP);

Rm:=GmC*(1-epsilonCx);
Rm:=max(Rm,GmC-qmN*(GmN+UmN));
Rm:=max(Rm,GmC-qmP*(GmP+UmP));

MN:=GmN+UmN-(GmC-Rm)/qmN;
MP:=GmP+UmP-(GmC-Rm)/qmP;

epsilonmC:=1-Rm/GmC;
epsilonmN:=1-MN/(GmN+UmN);
epsilonmP:=1-MP/(GmP+UmP);

if abs(flag2-1)<0.1 then {Liebig}
   begin
      UmC:=min(GmC*epsilonCx,min(qmN*(UmN+GmN),qmP*(UmP+GmP)));
      GmC:=UmC/epsilonCx;
      Rm:=GmC-UmC;
      GmN:=GmC*rmN*DN/(rmC*DC);
      GmP:=GmC*rmP*DP/(rmC*DC);
      MN:=GmN+UmN-(GmC-Rm)/qmN;
      MP:=GmP+UmP-(GmC-Rm)/qmP;
   end;
if abs(flag2-2)<0.1 then {Concurrent}
   begin
      UmC:=GmC*(UmN+GmN)*(UmP+GmP)*rhomM;
      GmC:=UmC/epsilonCx;
      Rm:=GmC-UmC;
      GmN:=GmC*rmN*DN/(rmC*DC);
      GmP:=GmC*rmP*DP/(rmC*DC);
      MN:=GmN+UmN-(GmC-Rm)/qmN;
      MP:=GmP+UmP-(GmC-Rm)/qmP;
   end;
if abs(flag2-3)<0.1 then {Adaptive}
   begin
      dum:=MN/UmN + MP/UmP - Rm/(qmN*UmN) - Rm/(qmP*UmP);
      etaC:=   1+GmC/(qmN*UmN)-GmN/UmN;
      etaC:=etaC+GmC/(UmP*qmP)-GmP/UmP;
      etaC:=(3-dum)/etaC;
      etaN:=etaC*(GmC/(qmN*UmN)-GmN/UmN)+ MN/UmN- Rm/(qmN*UmN);
      etaP:=etaC*(GmC/(UmP*qmP)-GmP/UmP)+ MP/UmP- Rm/(qmP*UmP);
      GmC:=etaC*GmC;
      GmN:=etaC*GmN;
      GmP:=etaC*GmP;
      UmN:=etaN*UmN;
      UmP:=etaP*UmP;
      Rm:=GmC*(1-epsilonCx);
      Rm:=max(Rm,GmC-qmN*(GmN+UmN));
      Rm:=max(Rm,GmC-qmP*(GmP+UmP));

      MN:=GmN+UmN-(GmC-Rm)/qmN;
      MP:=GmP+UmP-(GmC-Rm)/qmP;
   end;

Nmin:= MN-UmN;
Pmin:= MP-UMP;

QOC:=alphaOC*DC;
QON:=alphaON*DN;
QOP:=alphaOP*DP;
QN:=alphaN*EN;
QP:=alphaP*EP;
if CalculateDiscrete then
begin
// Add any discrete processes here
end; //discrete processes


{ Now calculate the derivatives of the state variables. If the holdConstant
  portion of the state variable is set to true then set the derivative equal to
  zero. }
if (tstat[1].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dBCdt := 0
else
 dBCdt:=PN-L_C;
 
if (tstat[2].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dBNdt := 0
else
 dBNdt:=UN+UNfix-L_N;
 
if (tstat[3].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dBPdt := 0
else
 dBPdt:=UP-L_P;
 
if (tstat[4].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dDCdt := 0
else
 dDCdt:=L_C-Rm-QOC;
 
if (tstat[5].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dDNdt := 0
else
 dDNdt:=L_N+UmN-QON-MN;
 
if (tstat[6].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dDPdt := 0
else
 dDPdt:=L_P+UmP-QOP-MP;
 
if (tstat[7].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dENdt := 0
else
 dENdt:=I_N+INF+MN-UN-UmN-QN;
 
if (tstat[8].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dEPdt := 0
else
 dEPdt:=I_P+MP-UP-UmP-QP;
 
if (tstat[9].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dVCdt := 0
else
 dVCdt:=dVCdt;
 
if (tstat[10].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dVNdt := 0
else
 dVNdt:=dVNdt;
 
if (tstat[11].HoldConstant) and (FmOptions.RunOptions.HoldStatesConstant) then
 dVPdt := 0
else
 dVPdt:=dVPdt;
 

{ Now that the calculations are complete, assign everything back into the arrays
  so the rest of the code can access the values calculated here. (Local variables
  are destroyed at the end of the procedure).

  Put the state variables back into the global arrays in case the state variable
  was manually changed in this procedure (e.g. discrete state variables or steady state
  calculations).   }
tstat[1].value := BC;
tstat[2].value := BN;
tstat[3].value := BP;
tstat[4].value := DC;
tstat[5].value := DN;
tstat[6].value := DP;
tstat[7].value := EN;
tstat[8].value := EP;
tstat[9].value := VC;
tstat[10].value := VN;
tstat[11].value := VP;

{  Put all process values into process variable array. The first numstate
  processes are the derivatives of the state variables (Calculated above).}
tproc[1].value := dBCdt;
tproc[2].value := dBNdt;
tproc[3].value := dBPdt;
tproc[4].value := dDCdt;
tproc[5].value := dDNdt;
tproc[6].value := dDPdt;
tproc[7].value := dENdt;
tproc[8].value := dEPdt;
tproc[9].value := dVCdt;
tproc[10].value := dVNdt;
tproc[11].value := dVPdt;

{ Now the remaining processes. Be sure to number the processes the same here as
  you did in the procedure counts above. }
tproc[ModelDef.numstate + 1].value := PN;
tproc[ModelDef.numstate + 2].value := UN;
tproc[ModelDef.numstate + 3].value := UP;
tproc[ModelDef.numstate + 4].value := UnC;
tproc[ModelDef.numstate + 5].value := UnN;
tproc[ModelDef.numstate + 6].value := UnP;
tproc[ModelDef.numstate + 7].value := L_C;
tproc[ModelDef.numstate + 8].value := L_N;
tproc[ModelDef.numstate + 9].value := L_P;
tproc[ModelDef.numstate + 10].value := Rm;
tproc[ModelDef.numstate + 11].value := MN;
tproc[ModelDef.numstate + 12].value := MP;
tproc[ModelDef.numstate + 13].value := GmC;
tproc[ModelDef.numstate + 14].value := epsilonmC;
tproc[ModelDef.numstate + 15].value := epsilonmN;
tproc[ModelDef.numstate + 16].value := epsilonmP;
tproc[ModelDef.numstate + 17].value := GmN;
tproc[ModelDef.numstate + 18].value := GmP;
tproc[ModelDef.numstate + 19].value := UmN;
tproc[ModelDef.numstate + 20].value := UmP;
tproc[ModelDef.numstate + 21].value := Nmin;
tproc[ModelDef.numstate + 22].value := Pmin;
tproc[ModelDef.numstate + 23].value := QOC;
tproc[ModelDef.numstate + 24].value := QON;
tproc[ModelDef.numstate + 25].value := QOP;
tproc[ModelDef.numstate + 26].value := QN;
tproc[ModelDef.numstate + 27].value := QP;
tproc[ModelDef.numstate + 28].value := BA;
tproc[ModelDef.numstate + 29].value := BW;
tproc[ModelDef.numstate + 30].value := q_N;
tproc[ModelDef.numstate + 31].value := q_P;
tproc[ModelDef.numstate + 32].value := AqN;
tproc[ModelDef.numstate + 33].value := AqP;
tproc[ModelDef.numstate + 34].value := LqN;
tproc[ModelDef.numstate + 35].value := LqP;
tproc[ModelDef.numstate + 36].value := RC;
tproc[ModelDef.numstate + 37].value := RN;
tproc[ModelDef.numstate + 38].value := RP;
tproc[ModelDef.numstate + 39].value := PSI;
tproc[ModelDef.numstate + 40].value := UNfix;
tproc[ModelDef.numstate + 41].value := PsiN;
tproc[ModelDef.numstate + 42].value := FNP;
tproc[ModelDef.numstate + 43].value := INF;
tproc[ModelDef.numstate + 44].value := phiN;
tproc[ModelDef.numstate + 45].value := phiP;
tproc[ModelDef.numstate + 46].value := MODEL;
tproc[ModelDef.numstate + 47].value := etaC;
tproc[ModelDef.numstate + 48].value := etaN;
tproc[ModelDef.numstate + 49].value := etaP;

end;  // End of processes procedure


       { Do not make any modifications to code below this line. }
{****************************************************************************}


{This function counts the parameters in all processes less than processnum.}
function ParCount(processnum:integer) : integer;
var
 NumberofParams, counter : integer;
begin
  NumberofParams := 0;
  for counter := ModelDef.numstate + 1 to processnum - 1 do
         NumberofParams := NumberofParams + proc[counter].parameters;
  ParCount := NumberofParams;
end; // end of parcount function

{ This procedure supplies the derivatives of the state variables to the
  integrator. Since the integrator deals only with the values of the variables
  and not there names, units or the state field HoldConstant, this procedure
  copies the state values into a temporary state array and copies the value of
  HoldConstant into the temporary state array and passes this temporary state
  array to the procedure processes. }
PROCEDURE derivs(t, drt:double; var tdrive:drivearray; var tpar:paramarray;
             var statevalue:yValueArray; VAR dydt:yValueArray);
var
   i:integer;
   tempproc:processarray;
   tempstate:statearray;
begin
   tempstate := stat;  // Copy names, units and HoldConstant to tempstate
  // Copy current values of state variables into tempstate
   for i := 1 to ModelDef.numstate do tempstate[i].value := statevalue[i];
  // Calculate the process values
   processes(t, drt, tdrive, tpar, tempstate, tempproc, false);
  // Put process values into dydt array to get passed back to the integrator.
   for i:= 1 to ModelDef.numstate do dydt[i]:=tempproc[i].value;
end;  // end of derivs procedure

end.
